
/*  
 *  Integrated and Modified by Sabina Chen for MIT's 6.810 Fall 2018 Final Project: Adaptive Accessories
 *  
--------Use This Sketch To------------------------------------------
1) Get user's live and changing BPM, beats per minute, in Arduino's native serial monitor via a PulseSensor
2) Smooths pulse sensor analog input by calculating the running average of the last ### readings 
3) BLOOM the flower at RESTING heart rate, or CLOSE the flower at ACTIVE heart rate. -> output PIN8
4) Waits 10 seconds after state change for flower to complete blooming/closing
AND
1) Determine whether it is light or dark in a room based on photoresistor input
2) BLOOM the flower if DARK, CLOSE the flower if LIGHT -> output PIN7
3) Waits 10 seconds after state change for flower to complete blooming/closing

Includes:
- ability to adjust threshold for resting heartbeat as well as light/dark value
- ability to adjust the number of past readings to be considered for the running average
--------------------------------------------------------------------*/

#define USE_ARDUINO_INTERRUPTS true    // Set-up low-level interrupts for most acurate BPM math.
#include <PulseSensorPlayground.h>     // Includes the PulseSensorPlayground Library.  

PulseSensorPlayground pulseSensor;  // Creates an instance of the PulseSensorPlayground object called "pulseSensor"

bool view_pulsesensor = true;
bool view_photoresistor = true;

// PulseSensor
const int PulseWire = 0;       // PulseSensor PURPLE WIRE connected to ANALOG PIN 0
const int LED13 = 13;          // The on-board Arduino LED, close to PIN 13.
const int PIN12 = 12;            // Pulse Sensor Muscle Wire Output
int pulsesensor_threshold = 550;    // Determine which Signal to "count as a beat" and which to ignore.
                                    // Use the "Getting Started Project" to fine-tune Threshold Value beyond default setting.
                                    // Otherwise leave the default "550" value. 
int bpm_previous = 300;             // Used to keep track user was previously at rest or active; detect changes in activity level
int resting_threshold = 75;         // Threshold to determine when a user is "active"

//  Photoresistor
const int photoresistor = 1;          // Photoresistor at Arduino analog pin 1
const int PIN8 = 8;                   // Photoresistor Muscle Wire Output
int photoresistor_value;              // Store value from photoresistor (0-1023)
int photoresistor_threshold = 250;    // Photoresistor threshold
int photoresistor_previous = 1000;    // Keep track if user was previously at rest or active
int photoresistor_tmp = photoresistor_previous;
                               
// Smoothing - calculates running average of the last ### readings of the analog input
const int numReadings = 7;
int readings[numReadings];
int readIndex = 0;
int total = 0;
int average_bpm = 0;

// Delay Counter (wait 10 seconds after change)
int counter = 0;
int counter2 = 0;


void setup() {   

  // initialize serial communication with computer
  Serial.begin(9600);          // For Serial Monitor

  // Configure the PulseSensor object, by assigning our variables to it. 
  pulseSensor.analogInput(PulseWire);   
  //pulseSensor.blinkOnPulse(LED13);       //blink Arduino's LED with heartbeat.
  pulseSensor.setThreshold(pulsesensor_threshold);   

  // Double-check the "pulseSensor" object was created and "began" seeing a signal. 
   if (pulseSensor.begin()) {
    Serial.println("Created a pulseSensor Object!");  //This prints one time at Arduino power-up,  or on Arduino reset.  
  }

  pinMode(photoresistor, INPUT);// Set pResistor - A0 pin as an input (optional)
  pinMode(LED13, OUTPUT);      // sets the digital pin as output
  pinMode(PIN8, OUTPUT);      // sets the digital pin as output

  // Smoothing - initialize all readings to 0
  for (int thisReading = 0; thisReading < numReadings; thisReading++) {
    readings[thisReading] = 0;
  }
  
}

void loop() {
  ////////////////////////////////////
  /////////   PulseSensor    /////////
  ////////////////////////////////////
  
  int myBPM = pulseSensor.getBeatsPerMinute();  // Calls function on our pulseSensor object that returns BPM as an "int"; "myBPM" hold this BPM value now. 

  if (pulseSensor.sawStartOfBeat()) {            // Constantly test to see if "a beat happened". 
    
    //////// GET BPM /////////

    if (view_pulsesensor) {
      Serial.print("BPM ♥: ");                      // Print phrase "BPM: " 
      Serial.println(myBPM);                        // Print the value inside of myBPM. 
    }

    //////// SMOOTHING /////////
    
    total = total - readings[readIndex];    // subtract the last reading
    readings[readIndex] = myBPM;            // read from sensor
    total = total + readings[readIndex];    // add the reading to the total
    readIndex = readIndex + 1;              // advance to the next position in the array

    // if we're at the end of the area, wrap around to the beginning
    if (readIndex >= numReadings) {
      readIndex = 0;
    }

    // calculate the average
    average_bpm = total / numReadings;

    if (view_pulsesensor) {
      Serial.print("CURRENT AVERAGE: ");
      Serial.println(average_bpm);
    }
 
    //////// DETERMINE RESTING vs ACTIVE /////////
    
    // if user working out
    if (average_bpm>resting_threshold && bpm_previous<resting_threshold && counter==0){
      if (view_pulsesensor) {
        Serial.print("Increase in physical activity! Average BPM is >");
        Serial.print(resting_threshold);
        Serial.println(" beats/min!");
      }
      bpm_previous = 300;

      // CLOSE FLOWER - active heartrate
      digitalWrite(PIN12, LOW); 
      Serial.print("WITHER");

      // after initial change of state, wait 10 seconds to allow time for muscle wire to change shape
      counter = counter + 1;
    }
    // if user at rest
    else if(average_bpm<resting_threshold && bpm_previous>resting_threshold && counter==0){
      if (view_pulsesensor) {
        Serial.print("At resting heartrate. Average BPM is <");
        Serial.print(resting_threshold);
        Serial.println(" beats/min!");
      }
      bpm_previous = 0;

      // BLOOM FLOWER - resting heartrate
      digitalWrite(PIN12, HIGH); 
      Serial.print("BLOOM");
      // after initial change of state, wait 10 seconds to allow time for muscle wire to change shape
      counter = counter + 1;
    }
  }

  //////// 10 SECOND DELAY FOR MUSCLE WIRE /////////

  // if 10 seconds have not passed yet, increase the counter 
  if (counter != 0 && counter != 500) {
    counter = counter + 1;
    if (view_pulsesensor) {
      if (counter % 50 == 0) {
        //Serial.println(counter);
      }
    }
  }

  // after 10 seconds, reset the counter 
  if (counter == 500) {
    counter = 0;
  }
  
  //////////////////////////////////////
  /////////   Photoresistor    /////////
  //////////////////////////////////////
  photoresistor_value = analogRead(photoresistor);

  //Serial.print("Value: ");
  //Serial.println(photoresistor_value);

  //////// DETERMINE LIGHT vs DARK /////////

  // if user is in LIGHT
  if (photoresistor_value>photoresistor_threshold && photoresistor_previous<photoresistor_threshold && counter2==0){
    if (view_photoresistor){
      Serial.println(photoresistor_value);
      Serial.println("LIGHTTTT - CLOSE!!!");
    }
    photoresistor_previous = photoresistor_tmp;

    // BLOOM FLOWER - light!
    digitalWrite(PIN8, LOW); 

    // after initial change of state, wait 10 seconds to allow time for muscle wire to change shape
    counter2 = counter2 + 1;
  }
  else if (photoresistor_value<photoresistor_threshold && photoresistor_previous>photoresistor_threshold && counter2==0){ 
    if (view_photoresistor) {
      Serial.println(photoresistor_value);
      Serial.println("ITS KINDA DARK - BLOOM!!!!!");
    }
    photoresistor_previous = 0;

    // WITHER FLOWER - dark..
    digitalWrite(PIN8, HIGH); 

    // after initial change of state, wait 10 seconds to allow time for muscle wire to change shape
    counter2 = counter2 + 1;
  }
  
  //////// 10 SECOND DELAY FOR MUSCLE WIRE /////////

  // if 10 seconds have not passed yet, increase the counter 
  if (counter2 != 0 && counter2 != 500) {
    counter2 = counter2 + 1;
    if (view_photoresistor) {
      if (counter2 % 50 == 0) {
        Serial.println(counter2);
      }
    }
  }

  // after 10 seconds, reset the counter 
  if (counter2 == 500) {
    counter2 = 0;
  }
 

  delay(20);                    // delay in between reads for stability

}
  
